local cow = {}

function cow:load(data) 
    self.visible = true
    self.mob = true
    self.physics = true
    self.abductable = true

    self.position = vec.new(data.x, data.y)
    self.targetPosition = vec.new(data.x, data.y)
    self.groundY = self.targetPosition.y
    self.velocity = vec.new()
    self.grounded = true

    self.moveRate = {2, 6}
    self.moveRadius = 50

    self.time = 0
    self.nextTime = random(self.moveRate[1], self.moveRate[2])

    self.moo = love.audio.newSource("data/art/moo.ogg", "static")
    self.mooTick = 0
    self.mooRate = {8, 20}
    self.nextMoo = random(self.mooRate[1], self.mooRate[2])

    self.abductionSound = love.audio.newSource("data/art/abduction.ogg", "static")
    self.abductionSound:setLooping(true)
    
    self.explosion = love.audio.newSource("data/art/cowexplode.ogg", "static")
    self.land = love.audio.newSource("data/art/land.ogg", "static")

    self.abductionInProgress = false
    self.dead = false
    self.buried = false

    self.particle = {
        abduction = fs.load("data/particles/abduction.lua")()[1],
        blood = fs.load("data/particles/blood.lua")()[1]
    }
    self.particle.blood.system:stop()
    self.particle.abduction.system:stop()

end

function cow:draw()
    -- Moo
    if not self.dead then
        self.mooTick = self.mooTick + love.timer.getDelta()
        if self.mooTick > self.nextMoo then
            
            self.moo:setVolume(0.5)
            self.moo:setPitch(1 + random(-0.05, 0.05))
            self.moo:setDirection(self.position.x, self.position.y, 0)
            self.moo:play()
            self.mooTick = 0
            self.nextMoo = random(self.mooRate[1], self.mooRate[2])
        end
    end

    if self.abductionInProgress then
        self.abductionSound:play()
    else
        self.abductionSound:stop()
    end

    if self.dead then
        self.abductionSound:stop()
    end

    if self.position.y > groundY then
        lg.setColor(color.shadow)
        lg.ellipse("fill", self.position.x, self.position.y + scale / 2, scale * 0.5, scale * 0.2)
    end
    lg.setColor(color.white)
    local scaleX = scale / 16
    if self.targetPosition.x > self.position.x then
        scaleX = -(scale / 16)
    end

    self.particle.abduction.system:moveTo(self.position.x - (scale / 8), self.position.y + scale / 2)
    self.particle.blood.system:moveTo(self.position.x - (scale / 8), self.position.y + (scale / 2))
    local sprite = cowSprite
    if self.dead then
        sprite = deadcowSprite
    end
    if self.buried then
        sprite = graveStone
        scaleX = scale / 16
    end
    lg.draw(sprite, floor(self.position.x), floor(self.position.y), 0, scaleX, scale / 16, 8, 8)
end

return cow