local ufo = {}

function ufo:load()
    self.visible = true
    self.control = false
    self.abductable = false
    self.physics = true

    self.position = vec.new(lg.getWidth() / 2, lg.getHeight() / 2)
    self.velocity = vec.new()
    self.acceleration = vec.new()
    self.horizontalAcceleration = scale * 4
    self.verticalAcceleration = scale * 12
    self.groundY = groundY
    self.grounded = false

    self.abductionInProgress = false
    self.abductionCount = 0
    self.crashWarningRange = scale

    self.particle = {
        thrust = require("data.particles.thrust")[1]
    }
    self.particle.thrust.system:stop()

    self.dead = false

    thrustSound:play()
    thrustSound:setVolume(0)

    self.thrust = 0
    self.targetThrust = 0

    self.explosion = anim.new("data/art/explosion.png", 512, 512, 0, false)
end

function ufo:draw()
    self.thrust = self.thrust + (self.targetThrust - self.thrust) * 10 * love.timer.getDelta()
    thrustSound:setVolume(self.thrust)

    self.explosion:update(love.timer.getDelta())
    lg.setColor(color.white)

    if not self.dead then
        lg.draw(ufoSprite, floor(self.position.x), floor(self.position.y), self.velocity.x * 0.001, scale / 16, scale / 16, 8, 8)

        local distanceToDeath = groundY - self.position.y
        if distanceToDeath < self.crashWarningRange then
            local normalDistanceToDeath = fmath.normal(self.position.y, groundY - self.crashWarningRange, groundY)
            lg.setColor(1, 0, 0, normalDistanceToDeath)
            lg.draw(ufoSprite, floor(self.position.x), floor(self.position.y), self.velocity.x * 0.001, scale / 16, scale / 16, 8, 8)
        end
    else
        local size = (scale * 2) 
        self.explosion:draw(self.position.x - (size / 2), self.position.y - (size / 2), size / 512, size / 512)
    end

    self.particle.thrust.system:moveTo(self.position.x + (scale / 2), self.position.y + (scale / 2))
end

return ufo