-- GLOBALS
lg = love.graphics
fs = love.filesystem
kb = love.keyboard
lm = love.mouse
random = math.random
noise = love.math.noise
sin = math.sin
cos = math.cos
abs = math.abs
floor = math.floor
f = string.format

function love.load()
    -- Löve setup
    lg.setDefaultFilter("nearest", "nearest")
    fs.setIdentity("Cow Game")

    -- Loading sprites
    bg = lg.newImage("data/art/bgNight.png")
    ufoSprite = lg.newImage("data/art/ufo.png")
    cowSprite = lg.newImage("data/art/cow.png")
    deadcowSprite = lg.newImage("data/art/deadcow.png")
    graveStone = lg.newImage("data/art/gravestone.png")
    bomb = lg.newImage("data/art/bomb.png")
    circle = lg.newImage("data/art/circle.png")
    logo = lg.newImage("data/art/logo.png")

    -- Sounds
    ambient = love.audio.newSource("data/art/ambient.ogg", "stream")
    ambient:setLooping(true)
    ambient:setVolume(0.03)

    explosion = love.audio.newSource("data/art/explosion.ogg", "static")
    abductionSound = love.audio.newSource("data/art/abduction.ogg", "static")
    abductionSound:setLooping(true)

    thrustSound = love.audio.newSource("data/art/spaceship.ogg", "static")
    thrustSound:setLooping(true)

    -- Loading libraries
    scene = require("data.lib.scene")
    fmath = require("data.lib.fmath")
    vec = require("data.lib.vec")
    ecs = require("data.lib.ecs")
    camera = require("data.lib.camera")
    anim = require("data.lib.anim")
    require("data.lib.color")

    -- Loading scenes
    scene:newScene("game", "data/scene/game.lua")
    scene:newScene("menu", "data/scene/menu.lua")
    scene:newScene("win", "data/scene/win.lua")

    -- Loading transitions
    scene:newTransition("swipe", "data/transition/swipe.lua")
    scene:newTransition("fade", "data/transition/fade.lua")

    -- Config
    config = {
        controls = {
            moveUp = "up",
            moveDown = "down",
            moveLeft = "left",
            moveRight = "right"
        }
    }

    -- Global variables 
    scale = lg.getHeight() * 0.1
    groundY = lg.getHeight() * 0.8
    abductionY = lg.getHeight() * 0.5
    maxVelocity = 3 * scale
    worldWidth = lg.getWidth() * 4

    -- Fonts
    font = {
        large = lg.newFont("data/font/PixelHigh.ttf", scale * 2),
        medium = lg.newFont("data/font/PixelMiddle.ttf", scale),
        small = lg.newFont("data/font/PixelSmall.ttf", scale),
        tiny = lg.newFont("data/font/PixelSmall.ttf", scale * 0.4)
    }

    
    -- Loading high score and progression
    highScore = 0
    if fs.getInfo("highscore") then
        highScore = fs.load("highscore")()
    else
        fs.write("highscore", "return "..highScore)
    end

    total = 5
    if fs.getInfo("total") then
        total = fs.load("total")()
    else
        fs.write("total", "return "..total)
    end

    ambient:play()
    scene:switch("menu")
end

function love.update(dt)
    scene:update(dt)
end

function love.draw()
    scene:draw()

    lg.setColor(color.red)
    lg.setFont(font.tiny)
    --lg.printf(love.timer.getFPS(), -12, 12, lg.getWidth(), "right")
end

function love.keypressed(key)
    if key == "escape" then love.event.push("quit") end
    scene:keypressed(key)
end